<?php

namespace Drupal\webform_booking\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\webform\Entity\Webform;
use Symfony\Component\HttpFoundation\JsonResponse;

class WebformBookingController extends ControllerBase {

public function getAvailableDays($webform_id, $date) {
  $webform = Webform::load($webform_id);
  $elements = $webform->getElementsDecodedAndFlattened();
  $config = [];

  foreach ($elements as $key => $element) {
    if (isset($element['#type']) && $element['#type'] === 'webform_booking') {
      $config = $element;
      break;
    }
  }

  $exclusionPatterns = $config['#exclusion_dates']; // e.g., ['2024-01-01|2024-01-05', '2024-12-25']
  $excludedWeekdays = $config['#excluded_weekdays']; // e.g., ['Sun', 'Sat']

  // Prepare the exclusion dates, including intervals
  $exclusionDates = [];
  foreach ($exclusionPatterns as $pattern) {
    if (strpos($pattern, '|') !== false) {
      list($start, $end) = explode('|', $pattern);
      $period = new \DatePeriod(
        new \DateTime($start),
        new \DateInterval('P1D'),
        (new \DateTime($end))->modify('+1 day')
      );
      foreach ($period as $date) {
        $exclusionDates[] = $date->format('Y-m-d');
      }
    } else {
      $exclusionDates[] = $pattern;
    }
  }

  // Parse the input date to get the start and end dates of its month
  $inputDate = new \DateTime($date);
  $startOfMonth = $inputDate->modify('first day of this month')->setTime(0, 0, 0);
  $endOfMonth = (clone $startOfMonth)->modify('last day of this month')->setTime(23, 59, 59);

  $interval = new \DateInterval('P1D'); // Interval of 1 day
  $dateRange = new \DatePeriod($startOfMonth, $interval, $endOfMonth);

  $days = [];

  foreach ($dateRange as $day) {
    $formattedDay = $day->format('Y-m-d');
    $today = new \DateTime();
    $weekday = $day->format('D');
    if ($day >= $today && !in_array($formattedDay, $exclusionDates) && isset($excludedWeekdays[$weekday]) && $excludedWeekdays[$weekday] === 0) {
        $days[] = $formattedDay;
    }
  }

    return new JsonResponse($days);
}

  public function getAvailableSlots($webform_id, $date) {

    $webform = Webform::load($webform_id);
    $elements = $webform->getElementsDecodedAndFlattened();
    $webform_key = NULL;
    $config = [];

    foreach ($elements as $key => $element) {
        if (isset($element['#type']) && $element['#type'] === 'webform_booking') {
            $config = $element;
            $webform_key = $key;
            break;
        }
    }
    $startTime = new \DateTime($config['#start_time']);
    $endTime = new \DateTime($config['#end_time']);
    $slotDuration = $config['#slot_duration'];

    $availableSlots = [];
    $currentTime = clone $startTime;

    // Generate slots
    while ($currentTime < $endTime) {
        $endSlotTime = clone $currentTime;
        $endSlotTime->add(new \DateInterval('PT' . $slotDuration . 'M')); // Add slot duration

        // Format for the slot time range
        $timeRange = $currentTime->format('H:i') . '-' . $endSlotTime->format('H:i');

        $slot = $date . ' ' . $currentTime->format('H:i');

        $slotStatus = $this->isSlotSelected($webform_id, $webform_key ,$slot) ? 'unavailable' : 'available';

        $availableSlots[] = ['time' => $timeRange, 'status' => $slotStatus];

        // Move to next slot
        $currentTime = clone $endSlotTime;
    }

    return new JsonResponse($availableSlots);
  }

  protected function isSlotSelected($webform_id, $webform_key, $slot) {
    $query = \Drupal::service('database')
      ->select('webform_submission_data', 'wsd')
      ->fields('wsd', array('sid'))
      ->condition('wsd.webform_id', $webform_id, '=')
      ->condition('wsd.name', $webform_key, '=')
      ->condition('wsd.value', $slot, '=')
      ->execute();
    $count = count($query->fetchAll(\PDO::FETCH_COLUMN));

    if ($count > 0) {
      return true;
    }
    return false;
  }
}
